package jcademo.rainbound.permtcp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;

import javax.resource.ResourceException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.work.Work;
import javax.resource.spi.work.WorkException;
import javax.resource.spi.work.WorkManager;

// accept connections and start new client handler for each connection
public class PermTCPWorker implements Work {
	private WorkManager wm;
	private MessageEndpointFactory mepf;
	private TestSpec as;
	private ServerSocket ss;
	private Socket s;
	private BufferedReader br;
	private PrintStream ps;
	private boolean superdone;
	private boolean done;
	public PermTCPWorker(WorkManager wm, MessageEndpointFactory mepf, TestSpec as, ServerSocket ss) throws ResourceException {
		this.wm = wm;
		this.mepf = mepf;
		this.as = as;
		this.ss = ss;
	}
	@Override
	public void run() {
		superdone = false;
		while(!superdone) {
			try {
				// accept connection
				s = ss.accept();
				br = new BufferedReader(new InputStreamReader(s.getInputStream(), "UTF-8"));
				ps = new PrintStream(s.getOutputStream(), false, "UTF-8");
				if(as.isDebug()) {
					System.out.println("Accepted connection");
				}
				done = false;
				while(!done) {
					// read id and numbers to add
					String id = br.readLine();
					if(id == null) break;
					String line = br.readLine();
					if(line == null) break;
					String[] lineparts = line.split(" ");
					int a = Integer.parseInt(lineparts[0]);
					int b = Integer.parseInt(lineparts[1]);
					// start new client handler
					wm.scheduleWork(new PermTCPClientWorker(mepf, as, ps, id, a, b));
				}
			} catch (WorkException e) {
				e.printStackTrace();
				done = true;
			} catch (IOException e) {
				e.printStackTrace();
				done = true;
			}
		}
	}
	@Override
	public void release() {
		superdone = true;
		done = true;
		try {
			ps.close();
			br.close();
			s.close();
			ss.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
